#
# Copyright (C) 2015 Jerome Kelleher <jerome.kelleher@well.ox.ac.uk>
#
# This file is part of msprime.
#
# msprime is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# msprime is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with msprime.  If not, see <http://www.gnu.org/licenses/>.
#
""""
Common code for the msprime test cases.
"""
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division

import collections
import random
import unittest

import _msprime


def setUp():
    # Make random tests reproducible.
    random.seed(210)


class PythonSparseTree(object):
    """
    Presents the same interface as the SparseTree object for testing. This
    is tightly coupled with the PythonTreeSequence object below which updates
    the internal structures during iteration.
    """
    def __init__(self):
        self.parent = {}
        self.children = {}
        self.time = {}
        self.left = 0
        self.right = 0
        self.root = 0
        self.index = -1
        self.sample_size = 0
        # We need a mutations function, so this name is taken.
        self.mutation_list = []

    @classmethod
    def from_sparse_tree(self, sparse_tree):
        ret = PythonSparseTree()
        ret.root = sparse_tree.get_root()
        ret.sample_size = sparse_tree.get_sample_size()
        ret.left, ret.right = sparse_tree.get_interval()
        ret.mutation_list = list(sparse_tree.mutations())
        ret.index = sparse_tree.get_index()
        # Traverse the tree and update the details as we go
        # We don't use the traversal method here because this
        # is used to test them.
        stack = [sparse_tree.get_root()]
        while len(stack) > 0:
            u = stack.pop()
            ret.time[u] = sparse_tree.get_time(u)
            if sparse_tree.is_internal(u):
                c = sparse_tree.get_children(u)
                stack.extend(c)
                for child in c:
                    ret.parent[child] = u
                ret.children[u] = c
        ret.parent[sparse_tree.get_root()] = 0
        assert ret == sparse_tree
        return ret

    def _preorder_nodes(self, u, l):
        l.append(u)
        if u in self.children:
            for c in self.children[u]:
                self._preorder_nodes(c, l)

    def nodes(self, root=None, order="preorder"):
        u = root
        if root is None:
            u = self.root
        if order == "preorder":
            l = []
            self._preorder_nodes(u, l)
            return iter(l)
        else:
            raise ValueError("order not supported")

    def get_sample_size(self):
        return self.sample_size

    def get_interval(self):
        return self.left, self.right

    def get_parent(self, node):
        return self.parent[node]

    def get_children(self, node):
        return self.children[node]

    def get_time(self, node):
        return self.time[node]

    def get_root(self):
        return self.root

    def get_index(self):
        return self.index

    def get_parent_dict(self):
        return self.parent

    def get_time_dict(self):
        return self.time

    def mutations(self):
        return iter(self.mutation_list)


class PythonTreeSequence(object):
    """
    A python implementation of the TreeDiffIterator algorithm.
    """
    def __init__(self, tree_sequence, breakpoints=None):
        self._tree_sequence = tree_sequence
        self._sample_size = tree_sequence.get_sample_size()
        self._breakpoints = breakpoints
        self._mutations = tree_sequence.get_mutations()

    def records(self):
        for j in range(self._tree_sequence.get_num_records()):
            yield self._tree_sequence.get_record(j, _msprime.MSP_ORDER_LEFT)

    def _diffs(self):
        left = 0
        used_records = collections.defaultdict(list)
        records_in = []
        for l, r, node, children, t in self.records():
            if l != left:
                # Out records must be sorted in reverse time order.
                records_out = sorted(used_records[left], key=lambda x: -x[2])
                yield l - left, records_out, records_in
                del used_records[left]
                records_in = []
                left = l
            used_records[r].append((node, children, t))
            records_in.append((node, children, t))
        records_out = sorted(used_records[left], key=lambda x: -x[2])
        yield r - left, records_out, records_in

    def _diffs_with_breaks(self):
        k = 1
        x = 0
        b = self._breakpoints
        for length, records_out, records_in in self._diffs():
            x += length
            yield b[k] - b[k - 1], records_out, records_in
            while self._breakpoints[k] != x:
                k += 1
                yield b[k] - b[k - 1], [], []
            k += 1

    def diffs(self, all_breaks=False):
        if all_breaks:
            return self._diffs_with_breaks()
        else:
            return self._diffs()

    def trees(self):
        st = PythonSparseTree()
        st.sample_size = self._tree_sequence.get_sample_size()
        st.left = 0
        st.time = {j: 0 for j in range(1, st.sample_size + 1)}
        for length, records_out, records_in in self.diffs():
            for node, children, t in records_out:
                del st.time[node]
                del st.children[node]
                for c in children:
                    del st.parent[c]
            for node, children, t in records_in:
                st.time[node] = t
                st.children[node] = children
                for c in children:
                    st.parent[c] = node
            # Insert the root
            root = 1
            while root in st.parent:
                root = st.parent[root]
            st.parent[root] = 0
            st.root = root
            st.index += 1
            st.right += length
            assert len(st.parent) == 2 * st.sample_size - 1
            # Add in all the mutations
            st.mutation_list = [
                (p, u) for (p, u) in self._mutations
                if st.left <= p < st.right
            ]
            yield st
            del st.parent[root]
            st.left = st.right


class MRCACalculator(object):
    """
    Class to that allows us to compute the nearest common ancestor of arbitrary
    nodes in an oriented forest.

    This is an implementation of Schieber and Vishkin's nearest common ancestor
    algorithm from TAOCP volume 4A, pg.164-167 [K11]_. Preprocesses the
    input tree into a sideways heap in O(n) time and processes queries for the
    nearest common ancestor between an arbitary pair of nodes in O(1) time.

    :param oriented_forest: the input oriented forest
    :type oriented_forest: list of integers
    """
    LAMBDA = 0

    def __init__(self, oriented_forest):
        self.__preprocess(oriented_forest)

    def __preprocess(self, oriented_forest):
        """
        Preprocess the oriented forest, so that we can answer mrca queries
        in constant time.
        """
        n = len(oriented_forest)
        child = [self.LAMBDA for i in range(n)]
        parent = [self.LAMBDA for i in range(n)]
        sib = [self.LAMBDA for i in range(n)]
        self.__lambda = [0 for i in range(n)]
        self.__pi = [0 for i in range(n)]
        self.__tau = [0 for i in range(n)]
        self.__beta = [0 for i in range(n)]
        self.__alpha = [0 for i in range(n)]
        for u in range(n):
            v = oriented_forest[u]
            sib[u] = child[v]
            child[v] = u
            parent[u] = v
        p = child[self.LAMBDA]
        n = 0
        self.__lambda[0] = -1
        while p != self.LAMBDA:
            notDone = True
            while notDone:
                n += 1
                self.__pi[p] = n
                self.__tau[n] = self.LAMBDA
                self.__lambda[n] = 1 + self.__lambda[n >> 1]
                if child[p] != self.LAMBDA:
                    p = child[p]
                else:
                    notDone = False
            self.__beta[p] = n
            notDone = True
            while notDone:
                self.__tau[self.__beta[p]] = parent[p]
                if sib[p] != self.LAMBDA:
                    p = sib[p]
                    notDone = False
                else:
                    p = parent[p]
                    if p != self.LAMBDA:
                        h = self.__lambda[n & -self.__pi[p]]
                        self.__beta[p] = ((n >> h) | 1) << h
                    else:
                        notDone = False
        # Begin the second traversal
        self.__lambda[0] = self.__lambda[n]
        self.__pi[self.LAMBDA] = 0
        self.__beta[self.LAMBDA] = 0
        self.__alpha[self.LAMBDA] = 0
        p = child[self.LAMBDA]
        while p != self.LAMBDA:
            notDone = True
            while notDone:
                a = (
                    self.__alpha[parent[p]] |
                    (self.__beta[p] & -self.__beta[p])
                )
                self.__alpha[p] = a
                if child[p] != self.LAMBDA:
                    p = child[p]
                else:
                    notDone = False
            notDone = True
            while notDone:
                if sib[p] != self.LAMBDA:
                    p = sib[p]
                    notDone = False
                else:
                    p = parent[p]
                    notDone = p != self.LAMBDA

    def get_mrca(self, x, y):
        """
        Returns the most recent common ancestor of the nodes x and y,
        or 0 if the nodes belong to different trees.

        :param x: the first node
        :type x: positive integer
        :param y: the second node
        :type y: positive integer
        :return: the MRCA of nodes x and y
        :type: non-negative integer
        """
        if self.__beta[x] <= self.__beta[y]:
            h = self.__lambda[self.__beta[y] & -self.__beta[x]]
        else:
            h = self.__lambda[self.__beta[x] & -self.__beta[y]]
        k = self.__alpha[x] & self.__alpha[y] & -(1 << h)
        h = self.__lambda[k & -k]
        j = ((self.__beta[x] >> h) | 1) << h
        if j == self.__beta[x]:
            xhat = x
        else:
            l = self.__lambda[self.__alpha[x] & ((1 << h) - 1)]
            xhat = self.__tau[((self.__beta[x] >> l) | 1) << l]
        if j == self.__beta[y]:
            yhat = y
        else:
            l = self.__lambda[self.__alpha[y] & ((1 << h) - 1)]
            yhat = self.__tau[((self.__beta[y] >> l) | 1) << l]
        if self.__pi[xhat] <= self.__pi[yhat]:
            z = xhat
        else:
            z = yhat
        return z


class MsprimeTestCase(unittest.TestCase):
    """
    Superclass of all tests msprime simulator test cases.
    """



#
# Copyright (C) 2015 Jerome Kelleher <jerome.kelleher@well.ox.ac.uk>
#
# This file is part of msprime.
#
# msprime is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# msprime is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with msprime.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Msprime is a reimplementation of Hudson's classical ms simulator for
modern datasets.
"""
from __future__ import print_function
from __future__ import division

from _msprime import get_gsl_version  # NOQA
from _msprime import get_hdf5_version  # NOQA

from msprime.trees import *  # NOQA

__version__ = "undefined"
try:
    from . import _version
    __version__ = _version.version
except ImportError:
    pass



#
# Copyright (C) 2015 Jerome Kelleher <jerome.kelleher@well.ox.ac.uk>
#
# This file is part of msprime.
#
# msprime is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# msprime is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with msprime.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Test cases for the high level interface to msprime.
"""
from __future__ import print_function
from __future__ import division

try:
    # We use the zip as iterator functionality here.
    from future_builtins import zip
except ImportError:
    # This fails for Python 3.x, but that's fine.
    pass

import math
import os
import random
import tempfile
import unittest
import xml.etree

import msprime
import tests


def get_pairwise_diversity(haplotypes):
    """
    Returns the value of pi for the specified haplotypes.
    """
    # Very simplistic algorithm...
    n = len(haplotypes)
    pi = 0
    for k in range(n):
        for j in range(k):
            for u, v in zip(haplotypes[j], haplotypes[k]):
                pi += u != v
    return 2 * pi / (n * (n - 1))


def sparse_tree_to_newick(st, precision):
    """
    Converts the specified sparse tree to an ms-compatible Newick tree.
    """
    branch_lengths = {}
    root = st.get_root()
    stack = [root]
    while len(stack) > 0:
        node = stack.pop()
        if st.is_internal(node):
            for child in st.get_children(node):
                stack.append(child)
                s = "{0:.{1}f}".format(
                    st.get_time(node) - st.get_time(child), precision)
                branch_lengths[child] = s
    return _build_newick(root, root, st, branch_lengths)


def _build_newick(node, root, tree, branch_lengths):
    c1, c2 = tree.get_children(node)
    if c1 != 0:
        s1 = _build_newick(c1, root, tree, branch_lengths)
        s2 = _build_newick(c2, root, tree, branch_lengths)
        if node == root:
            # The root node is treated differently
            s = "({0},{1});".format(s1, s2)
        else:
            s = "({0},{1}):{2}".format(
                s1, s2, branch_lengths[node])
    else:
        # Leaf node
        s = "{0}:{1}".format(node, branch_lengths[node])
    return s


class TestHarmonicNumber(unittest.TestCase):
    """
    Tests for the harmonic number calculation.
    """

    def test_harmonic_number(self):
        def H(n):
            return sum(1 / k for k in range(1, n + 1))
        for n in range(10, 1000, 100):
            self.assertAlmostEqual(msprime.harmonic_number(n), H(n), 1)


class TestMsCommandLine(tests.MsprimeTestCase):
    """
    Tests the output of the get_ms_command_line method.
    """
    def test_executable(self):
        sim = msprime.TreeSimulator(10)
        line = sim.get_ms_command_line()
        self.assertEqual(line[0], "ms")
        line = sim.get_ms_command_line("otherms")
        self.assertEqual(line[0], "otherms")

    def test_sample_size(self):
        for n in [2, 10, 100]:
            sim = msprime.TreeSimulator(n)
            self.assertEqual(sim.get_ms_command_line()[1], str(n))

    def test_recombination(self):
        for m in [10, 100, 1000]:
            for r in [0.125, 1.0, 10]:
                rho = r * (m - 1)
                sim = msprime.TreeSimulator(10)
                sim.set_num_loci(m)
                sim.set_scaled_recombination_rate(r)
                args = sim.get_ms_command_line()
                self.assertEqual(args[-3], "-r")
                self.assertEqual(args[-2], str(rho))
                self.assertEqual(args[-1], str(m))

    def test_mutation(self):
        for m in [1, 10, 100, 1000]:
            for u in [0.125, 1.0, 10]:
                mu = u * m
                sim = msprime.TreeSimulator(10)
                sim.set_scaled_recombination_rate(1.0)
                sim.set_num_loci(m)
                args = sim.get_ms_command_line(scaled_mutation_rate=u)
                self.assertEqual(args[-2], "-t")
                self.assertEqual(args[-1], str(mu))

    def test_trees(self):
        sim = msprime.TreeSimulator(10)
        self.assertIn("-T", sim.get_ms_command_line())
        self.assertIn("-T", sim.get_ms_command_line(output_trees=True))
        self.assertNotIn("-T", sim.get_ms_command_line(output_trees=False))
        self.assertIn("-T", sim.get_ms_command_line(scaled_mutation_rate=1.0))
        self.assertIn("-T", sim.get_ms_command_line(
            scaled_mutation_rate=1.0, output_trees=True))
        self.assertNotIn("-T", sim.get_ms_command_line(
            scaled_mutation_rate=1.0, output_trees=False))

    def test_num_replicates(self):
        for j in [1, 100, 1000]:
            sim = msprime.TreeSimulator(10)
            args = sim.get_ms_command_line(num_replicates=j)
            self.assertEqual(str(j), args[2])

    # TODO Test population models.


class HighLevelTestCase(tests.MsprimeTestCase):
    """
    Superclass of tests on the high level interface.
    """

    def verify_sparse_tree_mrcas(self, st):
        # Check the mrcas
        oriented_forest = [st.get_parent(j) for j in range(st.get_root() + 1)]
        mrca_calc = tests.MRCACalculator(oriented_forest)
        # We've done exhaustive tests elsewhere, no need to go
        # through the combinations.
        for j in range(1, st.get_root() + 1):
            mrca = st.get_mrca(1, j)
            self.assertEqual(mrca, mrca_calc.get_mrca(1, j))
            self.assertEqual(st.get_time(mrca), st.get_tmrca(1, j))

    def verify_sparse_tree_branch_lengths(self, st):
        for j in range(1, st.get_sample_size() + 1):
            u = j
            while st.get_parent(u) != 0:
                l = st.get_time(st.get_parent(u)) - st.get_time(u)
                self.assertGreater(l, 0.0)
                self.assertEqual(st.get_branch_length(u), l)
                u = st.get_parent(u)

    def verify_sparse_tree_structure(self, st):
        used_nodes = set()
        for j in range(1, st.get_sample_size() + 1):
            self.assertEqual(st.get_time(j), 0)
            # verify the path to root
            u = j
            times = []
            while st.get_parent(u) != 0:
                used_nodes.add(u)
                v = st.get_parent(u)
                times.append(st.get_time(v))
                self.assertGreaterEqual(st.get_time(v), 0.0)
                self.assertIn(u, st.get_children(v))
                u = v
            self.assertEqual(u, st.get_root())
            self.assertEqual(times, sorted(times))
        used_nodes.add(st.get_root())
        self.assertEqual(len(used_nodes), 2 * st.get_sample_size() - 1)
        # for every entry other than used_nodes we should have an empty row
        for j in range(st.get_root()):
            if j not in used_nodes:
                self.assertEqual(st.get_parent(j), 0)
                self.assertEqual(st.get_time(j), 0)
                for c in st.get_children(j):
                    self.assertEqual(c, 0)
        # To a top-down traversal, and make sure we meet all the leaves.
        stack = [st.get_root()]
        leaves = []
        while len(stack) > 0:
            u = stack.pop()
            self.assertNotEqual(u, 0)
            if st.is_leaf(u):
                leaves.append(u)
                self.assertEqual(st.get_children(u)[0], 0)
                self.assertEqual(st.get_children(u)[1], 0)
            else:
                for c in reversed(st.get_children(u)):
                    stack.append(c)
            # Check that we get the correct number of leaves at each
            # node.
            self.assertEqual(st.get_num_leaves(u), len(list(st.leaves(u))))
            self.assertEqual(st.get_num_tracked_leaves(u), 0)
        self.assertEqual(
            sorted(leaves), list(range(1, st.get_sample_size() + 1)))
        # Check the parent dict
        pi = st.get_parent_dict()
        self.assertEqual(len(pi), 2 * st.get_sample_size() - 1)
        self.assertEqual(pi[st.get_root()], 0)
        for k, v in pi.items():
            self.assertEqual(st.get_parent(k), v)

    def verify_sparse_tree(self, st):
        self.verify_sparse_tree_mrcas(st)
        self.verify_sparse_tree_branch_lengths(st)
        self.verify_sparse_tree_structure(st)

    def verify_sparse_trees(self, ts):
        pts = tests.PythonTreeSequence(ts.get_ll_tree_sequence())
        iter1 = ts.trees()
        iter2 = pts.trees()
        length = 0
        for st1, st2 in zip(iter1, iter2):
            self.assertEqual(st1.get_sample_size(), ts.get_sample_size())
            root = 1
            while st1.get_parent(root) != 0:
                root = st1.get_parent(root)
            self.assertEqual(root, st1.get_root())
            self.assertEqual(st1, st2)
            self.assertFalse(st1 != st2)
            l, r = st1.get_interval()
            self.assertEqual(l, length)
            self.assertGreaterEqual(l, 0)
            self.assertGreater(r, l)
            self.assertLessEqual(r, ts.get_num_loci())
            length += r - l
            self.verify_sparse_tree(st1)
        self.assertEqual(length, ts.get_num_loci())
        self.assertRaises(StopIteration, next, iter1)
        self.assertRaises(StopIteration, next, iter2)

    def verify_haplotype_statistics(self, ts):
        """
        Verifies the statistics calculated for the haplotypes
        in the specified tree sequence.
        """
        pi1 = ts.get_pairwise_diversity()
        pi2 = get_pairwise_diversity(list(ts.haplotypes()))
        self.assertAlmostEqual(pi1, pi2)
        self.assertGreaterEqual(pi1, 0.0)
        self.assertFalse(math.isnan(pi1))

    def verify_mutations(self, ts):
        """
        Verify the mutations on this tree sequence make sense.
        """
        self.verify_haplotype_statistics(ts)
        all_mutations = list(ts.mutations())
        # Mutations must be sorted by position
        self.assertEqual(
            all_mutations, sorted(all_mutations, key=lambda x: x[0]))
        self.assertEqual(len(all_mutations), ts.get_num_mutations())
        all_tree_mutations = []
        for st in ts.trees():
            tree_mutations = list(st.mutations())
            self.assertEqual(st.get_num_mutations(), len(tree_mutations))
            all_tree_mutations.extend(tree_mutations)
            for position, node in tree_mutations:
                left, right = st.get_interval()
                self.assertTrue(left <= position < right)
                self.assertNotEqual(st.get_parent(node), 0)
        self.assertEqual(all_tree_mutations, all_mutations)
        pts = tests.PythonTreeSequence(ts.get_ll_tree_sequence())
        iter1 = ts.trees()
        iter2 = pts.trees()
        for st1, st2 in zip(iter1, iter2):
            self.assertEqual(st1, st2)


class TestSingleLocusSimulation(HighLevelTestCase):
    """
    Tests on the single locus simulations.
    """
    def test_simple_cases(self):
        for n in range(2, 10):
            st = msprime.simulate_tree(n)
            self.verify_sparse_tree(st)
        for n in [11, 13, 19, 101]:
            st = msprime.simulate_tree(n)
            self.verify_sparse_tree(st)

    def test_error_cases(self):
        for n in [-100, -1, 0, 1]:
            self.assertRaises(ValueError, msprime.simulate_tree, n)
        for n in ["", None, "2", 2.2, 1e5]:
            self.assertRaises(TypeError, msprime.simulate_tree, n)

    def test_models(self):
        # Exponential growth of 0 and constant model should be identical.
        m1 = msprime.ExponentialPopulationModel(alpha=0.0, start_time=0.0)
        m2 = msprime.ConstantPopulationModel(size=1.0, start_time=0.0)
        for n in [2, 10, 100]:
            # TODO this _should_ be the same as running with no population
            # models, but it's not. need to investigate.
            st1 = msprime.simulate_tree(
                n, random_seed=1, population_models=[m1])
            st2 = msprime.simulate_tree(
                n, random_seed=1, population_models=[m2])
            self.assertEqual(st1, st2)
        # TODO add more tests!

    def test_initial_growth_rate(self):
        alpha = 5.0
        random_seed = 1234
        n = 10
        ts1 = msprime.simulate(
            n, random_seed=random_seed, population_models=[
                msprime.ExponentialPopulationModel(0.0, alpha)])
        sim = msprime.TreeSimulator(n)
        sim.set_random_seed(random_seed)
        sim.set_population_configurations([
            msprime.PopulationConfiguration(n, growth_rate=alpha)])
        sim.run()
        ts2 = sim.get_tree_sequence()
        self.assertEqual(list(ts1.records()), list(ts2.records()))

    def test_initial_pop_size(self):
        size = 5.0
        random_seed = 1234
        n = 10
        ts1 = msprime.simulate(
            n, random_seed=random_seed, population_models=[
                msprime.ConstantPopulationModel(0.0, size)])
        sim = msprime.TreeSimulator(n)
        sim.set_random_seed(random_seed)
        sim.set_population_configurations([
            msprime.PopulationConfiguration(n, initial_size=size)])
        sim.run()
        ts2 = sim.get_tree_sequence()
        self.assertEqual(list(ts1.records()), list(ts2.records()))

    def test_mixed_events(self):
        random_seed = 1234
        n = 10
        models = [
            msprime.ConstantPopulationModel(0.0, 2.0),
            msprime.ConstantPopulationModel(0.01, 5.0),
            msprime.ExponentialPopulationModel(0.5, 3.0)]
        ts1 = msprime.simulate(
            n, random_seed=random_seed, population_models=models)
        sim = msprime.TreeSimulator(n)
        sim.set_random_seed(random_seed)
        sim.set_population_configurations([
            msprime.PopulationConfiguration(n, initial_size=2)])
        sim.set_demographic_events([
            msprime.SizeChangeEvent(0.01, 5.0),
            msprime.GrowthRateChangeEvent(0.5, 3.0)])
        sim.run()
        ts2 = sim.get_tree_sequence()
        self.assertEqual(list(ts1.records()), list(ts2.records()))


class TestMultiLocusSimulation(HighLevelTestCase):
    """
    Tests on the single locus simulations.
    """

    def test_simple_cases(self):
        m = 1
        r = 0.1
        for n in range(2, 10):
            self.verify_sparse_trees(msprime.simulate(n, m, r))
        n = 4
        for m in range(1, 10):
            self.verify_sparse_trees(msprime.simulate(n, m, r))
        m = 100
        for r in [0.001, 0.01]:
            self.verify_sparse_trees(msprime.simulate(n, m, r))

    def test_error_cases(self):
        def f(n, m, r):
            return msprime.simulate(n, m, r)
        for n in [-100, -1, 0, 1]:
            self.assertRaises(ValueError, f, n, 1, 1.0)
        for n in ["", None, "2", 2.2, 1e5]:
            self.assertRaises(TypeError, f, n, 1, 1.0)


class TestTreeSimulator(HighLevelTestCase):
    """
    Runs tests on the underlying TreeSimulator object.
    """
    def verify_parameters(self, sim, tree_sequence):
        parameters = tree_sequence.get_parameters()
        self.assertIsInstance(parameters, dict)
        self.assertEqual(parameters["sample_size"], sim.get_sample_size())
        self.assertEqual(parameters["num_loci"], sim.get_num_loci())
        self.assertEqual(
            parameters["scaled_recombination_rate"],
            sim.get_scaled_recombination_rate())
        self.assertEqual(parameters["random_seed"], sim.get_random_seed())
        config = sim.get_configuration()
        self.assertEqual(config, parameters)

    def verify_dump_load(self, tree_sequence):
        """
        Dump the tree sequence and verify we can load again from the same
        file.
        """
        with tempfile.NamedTemporaryFile() as f:
            tree_sequence.dump(f.name)
            other = msprime.load(f.name)
        records = list(tree_sequence.records())
        other_records = list(other.records())
        self.assertEqual(records, other_records)

    def verify_simulation(self, n, m, r):
        """
        Verifies a simulation for the specified parameters.
        """
        sim = msprime.TreeSimulator(n)
        # TODO verify all the setters.
        self.assertEqual(sim.get_sample_size(), n)
        sim.set_scaled_recombination_rate(r)
        self.assertEqual(sim.get_scaled_recombination_rate(), r)
        sim.set_num_loci(m)
        self.assertEqual(sim.get_num_loci(), m)
        seed = 1
        sim.set_random_seed(seed)
        self.assertEqual(sim.get_random_seed(), seed)
        sim.run()
        self.assertEqual(sim.get_num_breakpoints(), len(sim.get_breakpoints()))
        self.assertGreater(sim.get_used_memory(), 0)
        self.assertGreater(sim.get_time(), 0)
        self.assertGreater(sim.get_num_avl_node_blocks(), 0)
        self.assertGreater(sim.get_num_segment_blocks(), 0)
        self.assertGreater(sim.get_num_node_mapping_blocks(), 0)
        self.assertGreater(sim.get_num_coalescence_record_blocks(), 0)
        self.assertGreater(sim.get_max_memory(), 0)
        tree_sequence = sim.get_tree_sequence()
        t = 0.0
        for _, _, _, _, time in tree_sequence.records():
            if time > t:
                t = time
        self.assertEqual(sim.get_time(), t)
        self.assertGreater(sim.get_num_common_ancestor_events(), 0)
        self.assertGreaterEqual(sim.get_num_recombination_events(), 0)
        self.assertGreaterEqual(sim.get_total_num_migration_events(), 0)
        self.assertGreaterEqual(sim.get_num_multiple_recombination_events(), 0)
        self.verify_sparse_trees(tree_sequence)
        self.verify_parameters(sim, tree_sequence)
        self.verify_dump_load(tree_sequence)

    def test_random_parameters(self):
        num_random_sims = 10
        for j in range(num_random_sims):
            n = random.randint(2, 100)
            m = random.randint(10, 100)
            r = random.random()
            self.verify_simulation(n, m, r)

    def test_perf_parameters(self):
        sim = msprime.TreeSimulator(10)
        # Before we call run, all perf parameters should be None
        self.assertIsNone(sim.get_avl_node_block_size())
        self.assertIsNone(sim.get_segment_block_size())
        self.assertIsNone(sim.get_node_mapping_block_size())
        self.assertIsNone(sim.get_coalescence_record_block_size())
        sim.run()
        self.assertGreater(sim.get_avl_node_block_size(), 0)
        self.assertGreater(sim.get_segment_block_size(), 0)
        self.assertGreater(sim.get_node_mapping_block_size(), 0)
        self.assertGreater(sim.get_coalescence_record_block_size(), 0)
        sim.reset()
        sim.set_avl_node_block_size(1)
        sim.set_segment_block_size(1)
        sim.set_node_mapping_block_size(1)
        sim.set_coalescence_record_block_size(1)
        self.assertEqual(sim.get_avl_node_block_size(), 1)
        self.assertEqual(sim.get_segment_block_size(), 1)
        self.assertEqual(sim.get_node_mapping_block_size(), 1)
        self.assertEqual(sim.get_coalescence_record_block_size(), 1)

    def test_bad_inputs(self):
        sim = msprime.TreeSimulator(10)
        for bad_type in ["xd", None, [], 4.4]:
            self.assertRaises(TypeError, msprime.TreeSimulator, bad_type)
            self.assertRaises(TypeError, sim.set_num_loci, bad_type)
        for bad_value in [-1, 0, 2**32]:
            self.assertRaises(ValueError, msprime.TreeSimulator, bad_value)
            self.assertRaises(ValueError, sim.set_num_loci, bad_value)
        self.assertRaises(ValueError, msprime.TreeSimulator, 1)
        self.assertRaises(ValueError, sim.set_scaled_recombination_rate, -1)


class TestHaplotypeGenerator(HighLevelTestCase):
    """
    Tests the haplotype generation code.
    """

    def verify_haplotypes(self, n, haplotype_strings):
        """
        Verify that the specified set of haplotypes are consistent.
        """
        self.assertEqual(len(haplotype_strings), n)
        for h in haplotype_strings:
            self.assertEqual(len(h), len(haplotype_strings[0]))
        # Examine each column; we must have a mixture of 0s and 1s
        for k in range(len(haplotype_strings[0])):
            zeros = 0
            ones = 0
            for j in range(n):
                b = haplotype_strings[j][k]
                zeros += b == '0'
                ones += b == '1'
            self.assertGreater(zeros, 0)
            self.assertGreater(ones, 0)
            self.assertEqual(zeros + ones, n)

    def verify_simulation(self, n, m, r, theta):
        """
        Verifies a simulation for the specified parameters.
        """
        ts = msprime.TreeSimulator(n)
        ts.set_scaled_recombination_rate(r)
        ts.set_num_loci(m)
        ts.run()
        tree_sequence = ts.get_tree_sequence()
        tree_sequence.generate_mutations(theta)
        haplotypes = list(tree_sequence.haplotypes())
        for h in haplotypes:
            self.assertEqual(len(h), tree_sequence.get_num_mutations())
        self.verify_haplotypes(n, haplotypes)

    def test_random_parameters(self):
        num_random_sims = 10
        for j in range(num_random_sims):
            n = random.randint(2, 100)
            m = random.randint(10, 1000)
            r = random.random()
            theta = random.uniform(0, 2)
            self.verify_simulation(n, m, r, theta)


class TestNewickConversion(HighLevelTestCase):
    """
    Test the newick tree generation code.
    """
    def verify_trees(self, tree_sequence, breakpoints):
        """
        Verifies that the specified tree is converted to Newick correctly.
        """
        def strip_tree(newick):
            """
            Strips all time information out of the specified newick tree.
            """
            s = newick.replace(":0", "")
            s = s.replace(":1", "")
            return s
        # We set the precision to 0 here to avoid problems that occur when
        # Python and C using different rounding strategies. This allows us
        # to remove the times completely, so we're just comparing the
        # structure of the trees.
        precision = 0
        old_trees = [
            (st.get_length(), sparse_tree_to_newick(st, precision))
            for st in tree_sequence.trees()]
        new_trees = list(tree_sequence.newick_trees(precision))
        self.assertEqual(len(new_trees), len(old_trees))
        for (l1, t1), (l2, t2) in zip(new_trees, old_trees):
            self.assertEqual(l1, l2)
            self.assertEqual(strip_tree(t1), strip_tree(t2))
        # TODO test the form of the trees when we're using breakpoints.

    def verify_all_breakpoints(self, tree_sequence, breakpoints):
        """
        Verifies that we get the correct list of trees when we use
        the all_breakpoints option for newick generation.
        """
        trees = list(tree_sequence.newick_trees(2, breakpoints))
        bp = [0] + breakpoints + [tree_sequence.get_num_loci()]
        self.assertEqual(len(trees), len(bp) - 1)
        j = 0
        s = 0
        for length, _ in trees:
            self.assertGreater(length, 0)
            self.assertEqual(s, bp[j])
            s += length
            j += 1
        self.assertEqual(s, tree_sequence.get_num_loci())
        pts = tests.PythonTreeSequence(
            tree_sequence.get_ll_tree_sequence(), bp)
        diffs = list(pts.diffs(all_breaks=True))
        self.assertEqual(len(diffs), len(trees))
        for j in range(1, len(diffs)):
            if len(diffs[j][1]) == 0:
                # If the list of diffs is empty, we should have the
                # same tree as the last one.
                self.assertEqual(trees[j][1], trees[j - 1][1])

    def test_simple_cases(self):
        cases = [
            (2, 1, 0),
            (2, 10, 0.1),
            (4, 10, 0.1),
            (10, 10, 0.1),
            (20, 1, 0),
            (20, 10, 0.1),
            (10, 50, 1.0),
        ]
        for n, m, r in cases:
            ts = msprime.TreeSimulator(n)
            ts.set_random_seed(1)
            ts.set_scaled_recombination_rate(r)
            ts.set_num_loci(m)
            ts.run()
            tree_sequence = ts.get_tree_sequence()
            breakpoints = ts.get_breakpoints()
            self.verify_trees(tree_sequence, breakpoints)
            self.verify_all_breakpoints(tree_sequence, breakpoints)

    def test_random_parameters(self):
        num_random_sims = 10
        for j in range(num_random_sims):
            n = random.randint(2, 100)
            m = random.randint(10, 1000)
            r = random.random()
            ts = msprime.TreeSimulator(n)
            ts.set_scaled_recombination_rate(r)
            ts.set_num_loci(m)
            ts.run()
            tree_sequence = ts.get_tree_sequence()
            breakpoints = ts.get_breakpoints()
            self.verify_trees(tree_sequence, breakpoints)
            self.verify_all_breakpoints(tree_sequence, breakpoints)


class TestTreeSequence(HighLevelTestCase):
    """
    Tests for the tree sequence object.
    """
    def get_example_tree_sequences(self):
        for n in [2, 3, 10, 100]:
            for m in [1, 2, 10, 100]:
                for rho in [0, 0.1, 10]:
                    yield msprime.simulate(n, m, rho)

    def test_sparse_trees(self):
        for ts in self.get_example_tree_sequences():
            self.verify_sparse_trees(ts)

    def test_mutations(self):
        all_zero = True
        for ts in self.get_example_tree_sequences():
            ts.generate_mutations(0)
            self.assertEqual(ts.get_num_mutations(), 0)
            for st in ts.trees():
                self.assertEqual(st.get_num_mutations(), 0)
            # choose a mutation rate that hopefully guarantees mutations,
            # but not too many.
            mu = 100 / ts.get_num_loci()
            ts.generate_mutations(mu)
            if ts.get_num_mutations() > 0:
                all_zero = False
                self.verify_mutations(ts)
            muts = [[], [(0, 1)], [(0, 1), (0, 2)]]
            for mutations in muts:
                ts.set_mutations(mutations)
                self.assertEqual(ts.get_num_mutations(), len(mutations))
                self.assertEqual(list(ts.mutations()), mutations)
                self.verify_mutations(ts)
        self.assertFalse(all_zero)

    def verify_tree_diffs(self, ts):
        pts = tests.PythonTreeSequence(ts.get_ll_tree_sequence())
        iter1 = ts.diffs()
        iter2 = pts.diffs()
        for t1, t2 in zip(iter1, iter2):
            self.assertEqual(t1, t2)
        self.assertRaises(StopIteration, next, iter1)
        self.assertRaises(StopIteration, next, iter2)

    def test_tree_diffs(self):
        for ts in self.get_example_tree_sequences():
            self.verify_tree_diffs(ts)

    def verify_tracked_leaves(self, ts):
        # Should be empty list by default.
        for tree in ts.trees():
            for u in tree.nodes():
                self.assertEqual(tree.get_num_tracked_leaves(u), 0)
        tracked_leaves = [1, 2]
        for tree in ts.trees(tracked_leaves):
            nu = [0 for j in range(ts.get_num_nodes() + 1)]
            for j in tracked_leaves:
                u = j
                while u != 0:
                    nu[u] += 1
                    u = tree.get_parent(u)
            for u, count in enumerate(nu):
                self.assertEqual(tree.get_num_tracked_leaves(u), count)

    def test_tracked_leaves(self):
        for ts in self.get_example_tree_sequences():
            self.verify_tracked_leaves(ts)


class TestSparseTree(HighLevelTestCase):
    """
    Some simple tests on the API for the sparse tree.
    """
    def get_tree(self):
        return msprime.simulate_tree(
            10, random_seed=1, scaled_mutation_rate=1)

    def test_str(self):
        t = self.get_tree()
        self.assertIsInstance(str(t), str)
        self.assertEqual(str(t), str(t.get_parent_dict()))

    def test_leaves(self):
        t = self.get_tree()
        n = t.get_sample_size()
        all_leaves = list(t.leaves(t.get_root()))
        self.assertEqual(sorted(all_leaves), list(range(1, n + 1)))
        for j in range(1, n + 1):
            self.assertEqual(list(t.leaves(j)), [j])

        def test_func(t, u):
            """
            Simple test definition of the traversal.
            """
            stack = [u]
            while len(stack) > 0:
                v = stack.pop()
                if t.is_internal(v):
                    for c in reversed(t.get_children(v)):
                        stack.append(c)
                else:
                    yield v
        for u in t.nodes():
            l1 = list(t.leaves(u))
            l2 = list(test_func(t, u))
            self.assertEqual(l1, l2)
            self.assertEqual(t.get_num_leaves(u), len(l1))

    def test_draw(self):
        t = self.get_tree()
        with tempfile.NamedTemporaryFile() as f:
            w = 123
            h = 456
            t.draw(f.name, w, h, show_times=True)
            self.assertGreater(os.path.getsize(f.name), 0)
            # Check some basic stuff about the SVG output.
            f.seek(0)
            root = xml.etree.ElementTree.fromstring(f.read())
            self.assertEqual(root.tag, "{http://www.w3.org/2000/svg}svg")
            width = int(root.attrib["width"])
            self.assertEqual(w, width)
            height = int(root.attrib["height"])
            self.assertEqual(h, height)

    def test_traversals(self):
        t1 = self.get_tree()
        t2 = tests.PythonSparseTree.from_sparse_tree(t1)
        self.assertEqual(list(t1.nodes()), list(t2.nodes()))
        self.assertEqual(list(t1.nodes()), list(t1.nodes(t1.get_root())))
        self.assertEqual(
            list(t1.nodes()),
            list(t1.nodes(t1.get_root(), "preorder")))
        for u in t1.nodes():
            self.assertEqual(list(t1.nodes(u)), list(t2.nodes(u)))
        self.assertRaises(ValueError, t1.nodes, None, "bad order")



"""
Code to stress the low-level API as much as possible to expose
any memory leaks or error handling issues.
"""
from __future__ import print_function
from __future__ import division

import unittest
import random

import tests.test_lowlevel as test_lowlevel
import tests.test_highlevel as test_highlevel


def main():
    while True:
        # We don't want any random variation in the amount of memory
        # used from test-to-test.
        random.seed(1)
        testloader = unittest.TestLoader()
        test_lowlevel.enable_h5py_tests = False
        suite = testloader.loadTestsFromModule(test_lowlevel)
        l = testloader.loadTestsFromModule(test_highlevel)
        suite.addTests(l)
        unittest.TextTestRunner(verbosity=0).run(suite)


if __name__ == "__main__":
    main()



#
# Copyright (C) 2015 Jerome Kelleher <jerome.kelleher@well.ox.ac.uk>
#
# This file is part of msprime.
#
# msprime is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# msprime is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with msprime.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Module responsible to generating and reading tree files.
"""
from __future__ import division
from __future__ import print_function

import json
import math
import random
import sys

try:
    import svgwrite
    _svgwrite_imported = True
except ImportError:
    _svgwrite_imported = False

import _msprime


class TreeDrawer(object):
    """
    A class to draw sparse trees in SVG format.
    """
    def __init__(self, tree, width, height, show_times):
        self._width = width
        self._height = height
        self._show_times = show_times
        self._x_scale = width / (tree.get_sample_size() + 2)
        t = tree.get_time(tree.get_root())
        # Leave a margin of 20px top and bottom
        y_padding = 20
        self._y_scale = (height - 2 * y_padding) / t
        self._tree = tree
        self._x_coords = {}
        self._y_coords = {}
        for u in tree.nodes():
            scaled_t = tree.get_time(u) * self._y_scale
            self._y_coords[u] = height - scaled_t - y_padding
        self._leaf_x = 1
        self._assign_x_coordinates(self._tree.get_root())
        self._mutations = []
        for pos, u in tree.mutations():
            x = self._x_coords[u], self._y_coords[u]
            v = tree.get_parent(u)
            y = self._x_coords[v], self._y_coords[v]
            z = (x[0] + y[0]) / 2, (x[1] + y[1]) / 2
            self._mutations.append(z)

    def write(self, path):
        """
        Writes the SVG description of this tree to the specified
        path.
        """
        dwg = svgwrite.Drawing(
            path, size=(self._width, self._height), debug=True)
        lines = dwg.add(dwg.g(id='lines', stroke='black'))
        labels = dwg.add(dwg.g(font_size=14))
        for u in self._tree.nodes():
            v = self._tree.get_parent(u)
            x = self._x_coords[u], self._y_coords[u]
            dwg.add(dwg.circle(center=x, r=3))
            dx = [0]
            dy = None
            if self._tree.is_leaf(u):
                dy = [20]
                dx = [-4]
            elif v == 0:
                dy = [-5]
            else:
                dx = [-20]
            labels.add(dwg.text(str(u), x, dx=dx, dy=dy))
            if self._show_times and self._tree.is_internal(u):
                dx[0] += 25
                labels.add(dwg.text(
                    "t = {:.2f}".format(self._tree.get_time(u)), x, dx=dx,
                    dy=dy)
                )
            if v != 0:
                y = self._x_coords[v], self._y_coords[v]
                lines.add(dwg.line(x, y))
        for x in self._mutations:
            dwg.add(dwg.rect(insert=x, size=(6, 6), fill="red"))
        dwg.save()

    def _assign_x_coordinates(self, node):
        """
        Assign x coordinates to all nodes underneath this node.
        """
        if self._tree.is_internal(node):
            children = self._tree.get_children(node)
            for c in children:
                self._assign_x_coordinates(c)
            # We now have x coords for both children
            c1 = self._x_coords[children[0]]
            c2 = self._x_coords[children[1]]
            a = min(c1, c2)
            b = max(c1, c2)
            self._x_coords[node] = (a + (b - a) / 2)
        else:
            self._x_coords[node] = self._leaf_x * self._x_scale
            self._leaf_x += 1


# TODO:
# - Pre, post, and inorder traversals of the nodes as iterators.
# - get_total_branch_length
# - Pickle and copy support
class SparseTree(object):
    """
    A SparseTree is a single tree in a :class:`.TreeSequence`. In a sparse tree
    for a sample of size :math:`n`, the leaves are nodes :math:`1` to :math:`n`
    inclusive and internal nodes are integers :math:`> n`. The value of these
    nodes is strictly increasing as we ascend the tree and the root of the tree
    is the node with the largest value that is reachable from  the leaves.
    Each node in the tree has a parent, which is non-zero for all non-root
    nodes reachable from the leaves. This value is obtained using the
    :meth:`.get_parent` method. The parent of the root node is 0. Similarly,
    each internal node has a pair of children, which are obtained using
    the :meth:`.get_children` method. Each node in the tree has a time
    associated with it in coalescent time units. This value is obtained using
    the :meth:`.get_time` method.

    Sparse trees are not intended to be instantiated directly, and are
    obtained as part of a :class:`.TreeSequence` using the
    :meth:`.trees` method.
    """
    def __init__(self, ll_sparse_tree):
        self._ll_sparse_tree = ll_sparse_tree

    def get_branch_length(self, u):
        """
        Returns the length of the branch (in time units) joining the
        specified node to its parent. This is equivalent to

        >>> tree.get_time(tree.get_parent(u)) - tree.get_time(u)

        Note that this is not related to the value returned by
        :meth:`.get_length`, which describes the length of the interval
        covered by the tree in genomic coordinates.

        :param int u: The node of interest.
        :return: The branch length from u to its parent.
        :rtype: int
        """
        return self.get_time(self.get_parent(u)) - self.get_time(u)

    def get_mrca(self, u, v):
        """
        Returns the most recent common ancestor of the specified nodes.

        :param int u: The first node.
        :param int v: The second node.
        :return: The most recent common ancestor of u and v.
        :rtype: int
        """
        return self._ll_sparse_tree.get_mrca(u, v)

    def get_tmrca(self, u, v):
        """
        Returns the time of the most recent common ancestor of the specified
        nodes. This is equivalent to::

            tree.get_time(tree.get_mrca(u, v))

        :param int u: The first node.
        :param int v: The second node.
        :return: The time of the most recent common ancestor of u and v.
        :rtype: float
        """
        return self.get_time(self.get_mrca(u, v))

    def get_parent(self, u):
        """
        Returns the parent of the specified node. Returns 0 if u is the
        root or is not a node in the current tree.

        :param int u: The node of interest.
        :return: The parent of u.
        :rtype: int
        """
        return self._ll_sparse_tree.get_parent(u)

    def get_children(self, u):
        """
        Returns the children of the specified node as a tuple (v, w). Returns
        the tuple (0, 0) if u is a leaf or is not a node in the current tree.

        :param int u: The node of interest.
        :return: The children of u as a pair of integers
        :rtype: tuple
        """
        return self._ll_sparse_tree.get_children(u)

    def get_time(self, u):
        """
        Returns the time of the specified node. Returns 0 if u is a leaf
        or is not a node in the current tree.

        :param int u: The node of interest.
        :return: The time of u.
        :rtype: float
        """
        return self._ll_sparse_tree.get_time(u)

    def is_internal(self, u):
        """
        Returns True if the specified node is not a leaf.

        :param int u: The node of interest.
        :return: True if u is not a leaf node.
        :rtype: bool
        """
        return not self.is_leaf(u)

    def is_leaf(self, u):
        """
        Returns True if the specified node is a leaf. A node :math:`u` is a
        leaf if :math:`1 \leq u \leq n` for a sample size :math:`n`.

        :param int u: The node of interest.
        :return: True if u is a leaf node.
        :rtype: bool
        """
        return 1 <= u <= self.get_sample_size()

    def get_root(self):
        """
        Returns the root of this tree.

        :return: The root node.
        :rtype: int
        """
        return self._ll_sparse_tree.get_root()

    def get_index(self):
        """
        Returns the index this tree occupies in the parent tree sequence.
        This index is zero based, so the first tree in the sequence has index
        0.

        :return: The index of this tree.
        :rtype: int
        """
        return self._ll_sparse_tree.get_index()

    def get_interval(self):
        """
        Returns the coordinates of the genomic interval that this tree
        represents the history of. The interval is returned as a tuple
        :math:`(l, r)` and is a half-open interval such that the left
        coordinate is inclusive and the right coordinate is exclusive. This
        tree therefore applies to all genomic locations :math:`x` such that
        :math:`l \leq x < r`.

        :return: A tuple (l, r) representing the left-most (inclusive)
            and right-most (exclusive) coordinates of the genomic region
            covered by this tree.
        :rtype: tuple
        """
        return (
            self._ll_sparse_tree.get_left(), self._ll_sparse_tree.get_right()
        )

    def get_length(self):
        """
        Returns the length of the genomic interval that this tree represents.
        This is defined as :math:`r - l`, where :math:`(l, r)` is the genomic
        interval returned by :meth:`.get_interval`.

        :return: The length of the genomic interval covered by this tree.
        :rtype: int
        """
        l, r = self.get_interval()
        return r - l

    def get_sample_size(self):
        """
        Returns the sample size for this tree. This is the number of leaf
        nodes in the tree.

        :return: The number of leaf nodes in the tree.
        :rtype: int
        """
        return self._ll_sparse_tree.get_sample_size()

    def draw(self, path, width=200, height=200, show_times=False):
        """
        Draws a representation of this tree to the specified path in SVG
        format.

        :param str path: The path to the file to write the SVG.
        :param int width: The width of the image in pixels.
        :param int height: The height of the image in pixels.
        :param bool show_times: If True, show time labels at each internal
            node.
        """
        if not _svgwrite_imported:
            raise ImportError(
                "svgwrite is not installed. try `pip install svgwrite`")
        td = TreeDrawer(self, width, height, show_times)
        td.write(path)

    def get_num_mutations(self):
        """
        Returns the number of mutations on this tree.

        :return: The number of mutations on this tree.
        :rtype: int
        """
        return self._ll_sparse_tree.get_num_mutations()

    def mutations(self):
        """
        Returns an iterator over the mutations on this tree. Each mutation
        is represented as a tuple (position, node), and mutations
        returned in increasing order of position.

        :return: The mutations in this tree.
        :rtype: iterator
        """
        return iter(self._ll_sparse_tree.get_mutations())

    def leaves(self, u):
        """
        Returns an iterator over all the leaves in this tree underneath
        the specified node.

        :param int u: The node of interest.
        :return: An iterator over all leaves in the subtree rooted at u.
        :rtype: iterator
        """
        return _msprime.LeafListIterator(self._ll_sparse_tree, u)

    def get_num_leaves(self, u):
        """
        Returns the number of leaves in this tree underneath the specified
        node. This is a constant time operation.

        :param int u: The node of interest.
        :return: The number of leaves in the subtree rooted at u.
        :rtype: int
        """
        return self._ll_sparse_tree.get_num_leaves(u)

    def get_num_tracked_leaves(self, u):
        """
        Returns the number of leaves in the set specified in the
        ``tracked_leaves`` parameter of the :meth:`msprime.TreeSequence.trees`
        method underneath the specified node. This is a constant time
        operation.

        :param int u: The node of interest.
        :return: The number of leaves within the set of tracked leaves in
            the subtree rooted at u.
        :rtype: int
        """
        return self._ll_sparse_tree.get_num_tracked_leaves(u)

    def _preorder_traversal(self, u):
        stack = [u]
        while len(stack) > 0:
            v = stack.pop()
            if self.is_internal(v):
                stack.extend(reversed(self.get_children(v)))
            yield v

    def nodes(self, root=None, order="preorder"):
        """
        Returns an iterator over the nodes in this tree. If the root parameter
        is provided, iterate over the nodes in the subtree rooted at this
        node. If this is None, iterate over all nodes. If the order parameter
        is provided, iterate over the nodes in required tree traversal order.

        :param int root: The root of the subtree we are traversing.
        :param str order: The traversal ordering. Currently only 'preorder'
            is supported.
        :rtype: iterator
        """
        u = self.get_root() if root is None else root
        if order == "preorder":
            return self._preorder_traversal(u)
        else:
            raise ValueError(
                "Traversal ordering '{}' not supported".format(order))

    def get_parent_dict(self):
        pi = {}
        for j in range(1, self.get_sample_size() + 1):
            u = j
            while u != 0 and u not in pi:
                pi[u] = self.get_parent(u)
                u = pi[u]
        return pi

    def get_time_dict(self):
        tau = {}
        for j in range(1, self.get_sample_size() + 1):
            u = j
            while u != 0 and u not in tau:
                tau[u] = self.get_time(u)
                u = self.get_parent(u)
        return tau

    def __str__(self):
        return str(self.get_parent_dict())

    def __eq__(self, other):
        return (
            self.get_sample_size() == other.get_sample_size() and
            self.get_parent_dict() == other.get_parent_dict() and
            self.get_time_dict() == other.get_time_dict() and
            self.get_interval() == other.get_interval() and
            self.get_root() == other.get_root() and
            self.get_index() == other.get_index() and
            list(self.mutations()) == list(other.mutations()))

    def __ne__(self, other):
        return not self.__eq__(other)


def simulate(
        sample_size, num_loci=1, scaled_recombination_rate=0.0,
        scaled_mutation_rate=None,
        population_models=[], random_seed=None, max_memory=None):
    """
    Simulates the coalescent with recombination under the specified model
    parameters and returns the resulting :class:`.TreeSequence`.

    :param int sample_size: The number of individuals in our sample.
    :param int num_loci: The length of the simulated region in
        discrete non-recombining loci.
    :param float scaled_recombination_rate: The rate of recombination
        between adjacent loci per :math:`4N_0` generations.
    :param float scaled_mutation_rate: The rate of mutation
        per locus per :math:`4N_0` generations.
    :param list population_models: The list of :class:`.PopulationModel`
        instances describing the demographic history of the population.
    :param int random_seed: The random seed. If this is `None`, a
        random seed will be automatically generated.
    :param int,str max_memory: The maximum amount of memory used
        during the simulation. If this is exceeded, the simulation will
        terminate with a :class:`LibraryError` exception. By default
        this is not set, and no memory limits are imposed.
    :return: The :class:`.TreeSequence` object representing the results
        of the simulation.
    :rtype: :class:`.TreeSequence`
    """
    sim = TreeSimulator(sample_size)
    sim.set_num_loci(num_loci)
    sim.set_scaled_recombination_rate(scaled_recombination_rate)
    sim.set_random_seed(random_seed)
    sim.set_max_memory(max_memory)
    # Reinterpret the population models in terms of the new interface.
    # This will be deprecated in later releases in favour of a direct
    # approach.
    initial_size = 1
    initial_growth_rate = 0
    demographic_events = []
    for model in population_models:
        if isinstance(model, ConstantPopulationModel):
            if model.start_time == 0:
                initial_size = model.size
            else:
                demographic_events.append(SizeChangeEvent(
                    model.start_time, model.size))
        elif isinstance(model, ExponentialPopulationModel):
            if model.start_time == 0:
                initial_growth_rate = model.alpha
            else:
                demographic_events.append(GrowthRateChangeEvent(
                    model.start_time, model.alpha))
        else:
            raise TypeError("Arguments must be PopulationModel instances")
    sim.set_population_configurations([PopulationConfiguration(
        sample_size, initial_size, initial_growth_rate)])
    sim.set_demographic_events(demographic_events)
    sim.run()
    tree_sequence = sim.get_tree_sequence()
    if scaled_mutation_rate is not None:
        tree_sequence.generate_mutations(scaled_mutation_rate, random_seed)
    return tree_sequence


def simulate_tree(
        sample_size, scaled_mutation_rate=None, population_models=[],
        random_seed=None, max_memory=None):
    """
    Simulates the coalescent at a single locus for the specified sample size
    under the specified list of population models. Returns a
    :class:`.SparseTree` representing the results.

    :param int sample_size: The number of individuals in our sample.
    :param float scaled_mutation_rate: The rate of mutation
        per :math:`4N_0` generations.
    :param int random_seed: The random seed. If this is `None`, a
        random seed will be automatically generated.
    :param list population_models: The list of :class:`.PopulationModel`
        instances describing the demographic history of the population.
    :param int,str max_memory: The maximum amount of memory used
        during the simulation. If this is exceeded, the simulation will
        terminate with a :class:`LibraryError` exception. By default
        this is not set, and no memory limits are imposed.
    :return: The :class:`.SparseTree` object representing the results
        of the simulation.
    :rtype: :class:`.SparseTree`
    """
    tree_sequence = simulate(
        sample_size, scaled_mutation_rate=scaled_mutation_rate,
        population_models=population_models, random_seed=random_seed,
        max_memory=max_memory)
    return next(tree_sequence.trees())


def load(path):
    """
    Loads a tree sequence from the specified file path. This
    file must be in the HDF5 file format produced by the
    :meth:`msprime.TreeSequence.dump` method.

    :param str path: The file path to write the TreeSequence to.
    :return: The tree sequence object containing the information
        stored in the specified file path.
    :rtype: :class:`msprime.TreeSequence`
    """
    return TreeSequence.load(path)


class TreeSimulator(object):
    """
    Class to simulate trees under the standard neutral coalescent with
    recombination.
    """
    def __init__(self, sample_size):
        if not isinstance(sample_size, int):
            raise TypeError("Sample size must be an integer")
        if sample_size < 2:
            raise ValueError("Sample size must be >= 2")
        if sample_size >= 2**32:
            raise ValueError("sample_size must be < 2**32")
        self._sample_size = sample_size
        self._scaled_recombination_rate = 0.0
        self._num_loci = 1
        self._migration_matrix = None
        self._population_configurations = None
        self._demographic_events = []
        self._random_seed = None
        self._segment_block_size = None
        self._avl_node_block_size = None
        self._node_mapping_block_size = None
        self._coalescence_record_block_size = None
        self._max_memory = None
        self._ll_sim = None

    def get_sample_size(self):
        return self._sample_size

    def get_scaled_recombination_rate(self):
        return self._scaled_recombination_rate

    def get_migration_matrix(self):
        return self._migration_matrix

    def get_num_loci(self):
        return self._num_loci

    def get_random_seed(self):
        return self._random_seed

    def get_population_configurations(self):
        return self._population_configurations

    def get_sample_configuration(self):
        return [conf.sample_size for conf in self._population_configurations]

    def get_demographic_events(self):
        return self._demographic_events

    def get_num_breakpoints(self):
        return self._ll_sim.get_num_breakpoints()

    def get_breakpoints(self):
        return self._ll_sim.get_breakpoints()

    def get_used_memory(self):
        return self._ll_sim.get_used_memory()

    def get_time(self):
        return self._ll_sim.get_time()

    def get_avl_node_block_size(self):
        return self._avl_node_block_size

    def get_coalescence_record_block_size(self):
        return self._coalescence_record_block_size

    def get_node_mapping_block_size(self):
        return self._node_mapping_block_size

    def get_segment_block_size(self):
        return self._segment_block_size

    def get_num_avl_node_blocks(self):
        return self._ll_sim.get_num_avl_node_blocks()

    def get_num_coalescence_record_blocks(self):
        return self._ll_sim.get_num_coalescence_record_blocks()

    def get_num_node_mapping_blocks(self):
        return self._ll_sim.get_num_node_mapping_blocks()

    def get_num_segment_blocks(self):
        return self._ll_sim.get_num_segment_blocks()

    def get_num_common_ancestor_events(self):
        return self._ll_sim.get_num_common_ancestor_events()

    def get_num_recombination_events(self):
        return self._ll_sim.get_num_recombination_events()

    def get_num_populations(self):
        return len(self._population_configurations)

    def get_num_migration_events(self):
        N = self.get_num_populations()
        matrix = [[0 for j in range(N)] for k in range(N)]
        flat = self._ll_sim.get_num_migration_events()
        for j in range(N):
            for k in range(N):
                matrix[j][k] = flat[j * N + k]
        return matrix

    def get_total_num_migration_events(self):
        return sum(self._ll_sim.get_num_migration_events())

    def get_num_multiple_recombination_events(self):
        return self._ll_sim.get_num_multiple_recombination_events()

    def get_max_memory(self):
        return self._ll_sim.get_max_memory()

    def get_configuration(self):
        return json.loads(self._ll_sim.get_configuration_json())

    def set_num_loci(self, num_loci):
        if not isinstance(num_loci, int):
            raise TypeError("num_loci must be an integer")
        if num_loci < 1:
            raise ValueError("Positive number of loci required")
        if num_loci >= 2**32:
            raise ValueError("num_loci must be < 2**32")
        self._num_loci = num_loci

    def set_scaled_recombination_rate(self, scaled_recombination_rate):
        if scaled_recombination_rate < 0:
            raise ValueError("Recombination rate cannot be negative")
        self._scaled_recombination_rate = scaled_recombination_rate

    def set_migration_matrix(self, migration_matrix):
        self._migration_matrix = migration_matrix

    def set_population_configurations(self, population_configurations):
        self._population_configurations = population_configurations

    def set_demographic_events(self, demographic_events):
        self._demographic_events = demographic_events

    def set_random_seed(self, random_seed):
        self._random_seed = random_seed

    def set_segment_block_size(self, segment_block_size):
        self._segment_block_size = segment_block_size

    def set_avl_node_block_size(self, avl_node_block_size):
        self._avl_node_block_size = avl_node_block_size

    def set_node_mapping_block_size(self, node_mapping_block_size):
        self._node_mapping_block_size = node_mapping_block_size

    def set_coalescence_record_block_size(self, coalescence_record_block_size):
        self._coalescence_record_block_size = coalescence_record_block_size

    def set_max_memory(self, max_memory):
        """
        Sets the approximate maximum memory used by the simulation
        to the specified value.  This can be suffixed with
        K, M or G to specify units of Kibibytes, Mibibytes or Gibibytes.
        """
        if max_memory is None:
            self._max_memory = max_memory
        else:
            s = max_memory
            d = {"K": 2**10, "M": 2**20, "G": 2**30}
            multiplier = 1
            value = s
            if s.endswith(tuple(d.keys())):
                value = s[:-1]
                multiplier = d[s[-1]]
            n = int(value)
            self._max_memory = n * multiplier

    def _set_environment_defaults(self):
        """
        Sets sensible default values for the memory usage parameters.
        """
        # Set the block sizes using our estimates.
        n = self._sample_size
        m = self._num_loci
        if self._population_configurations is None:
            self._population_configurations = [PopulationConfiguration(n)]
        N = len(self._population_configurations)
        if self._migration_matrix is None:
            self._migration_matrix = [
                [0.0 for j in range(N)] for k in range(N)]
        if self._demographic_events is None:
            self._demographic_events = []
        rho = 4 * self._scaled_recombination_rate * (m - 1)
        num_trees = min(m // 2, rho * harmonic_number(n - 1))
        b = 10  # Baseline maximum
        num_trees = max(b, int(num_trees))
        num_avl_nodes = max(b, 4 * n + num_trees)
        # TODO This is total guesswork. We need to plot this for a range
        # of values and see what a good approximation is.
        num_segments = max(b, int(math.log(n) * rho))
        if self._avl_node_block_size is None:
            self._avl_node_block_size = num_avl_nodes
        if self._segment_block_size is None:
            self._segment_block_size = num_segments
        if self._node_mapping_block_size is None:
            self._node_mapping_block_size = num_trees
        if self._coalescence_record_block_size is None:
            memory = 16 * 2**10  # 16M
            # Each coalescence record is 32bytes
            self._coalescence_record_block_size = memory // 32
        if self._random_seed is None:
            self._random_seed = random.randint(0, 2**31 - 1)
        if self._max_memory is None:
            self._max_memory = sys.maxsize  # Unlimited

    def run(self):
        """
        Runs the simulation until complete coalescence has occurred.
        """
        assert self._ll_sim is None
        self._set_environment_defaults()
        # We flatten the migration matrix for the low-level interface.
        N = len(self._population_configurations)
        ll_migration_matrix = [0 for j in range(N**2)]
        for j in range(N):
            for k in range(N):
                ll_migration_matrix[j * N + k] = self._migration_matrix[j][k]
        ll_population_configuration = [
            conf.get_ll_representation()
            for conf in self._population_configurations]
        ll_demographic_events = [
            event.get_ll_representation(N)
            for event in self._demographic_events]
        self._ll_sim = _msprime.Simulator(
            sample_size=self._sample_size,
            num_loci=self._num_loci,
            migration_matrix=ll_migration_matrix,
            population_configuration=ll_population_configuration,
            demographic_events=ll_demographic_events,
            scaled_recombination_rate=self._scaled_recombination_rate,
            random_seed=self._random_seed,
            max_memory=self._max_memory,
            segment_block_size=self._segment_block_size,
            avl_node_block_size=self._avl_node_block_size,
            node_mapping_block_size=self._node_mapping_block_size,
            coalescence_record_block_size=self._coalescence_record_block_size)
        self._ll_sim.run()

    def get_tree_sequence(self):
        """
        Returns a TreeSequence representing the state of the simulation.
        """
        ll_tree_sequence = _msprime.TreeSequence()
        ll_tree_sequence.create(self._ll_sim)
        return TreeSequence(ll_tree_sequence)

    def reset(self):
        """
        Resets the simulation so that we can perform another replicate.
        """
        self._ll_sim = None

    def get_ms_command_line(
            self, executable="ms", num_replicates=1, output_trees=True,
            scaled_mutation_rate=None):
        """
        Returns an command line for ms that is equivalent to the parameters
        for this simulator.

        :param str executable: The path to the ms-compatible binary.
        :param int num_relicates: The number of replicates to simulate.
        :param float scaled_mutation_rate: The rate of mutation
            per :math:`4N_0` generations.
        :return: A list of command line arguments that can be used to invoke
            ms with equivalent parameters to this simulator.
        :rtype: list
        """
        self._set_environment_defaults()
        m = self._num_loci
        rho = self.get_scaled_recombination_rate() * (m - 1)
        args = [executable, str(self._sample_size), str(num_replicates)]
        if output_trees:
            args += ["-T"]
        if m > 1:
            args += ["-r", str(rho), str(m)]
        if scaled_mutation_rate is not None:
            mu = scaled_mutation_rate * m
            args += ["-t", str(mu)]
        for conf in self._population_configurations:
            if conf.growth_rate > 0:
                args.extend(["-G", str(conf.growth_rate)])
        if len(self._population_configurations) > 1:
            # TODO add -I arguments
            assert False
        for event in self._demographic_events:
            args.extend(event.get_ms_arguments())
        return args


class TreeSequence(object):
    """
    A TreeSequence represents the information generated in a coalescent
    simulation. This includes all the trees across the simulated region,
    along with the mutations (if any are present).
    """

    def __init__(self, ll_tree_sequence):
        self._ll_tree_sequence = ll_tree_sequence

    def get_ll_tree_sequence(self):
        return self._ll_tree_sequence

    @classmethod
    def load(cls, path):
        ts = _msprime.TreeSequence()
        ts.load(path)
        return TreeSequence(ts)

    def get_parameters(self):
        return json.loads(self._ll_tree_sequence.get_simulation_parameters())

    def get_mutations(self):
        # TODO should we provide this???
        return self._ll_tree_sequence.get_mutations()

    def newick_trees(self, precision=3, breakpoints=None):
        # TODO document this method.
        iterator = _msprime.NewickConverter(self._ll_tree_sequence, precision)
        if breakpoints is None:
            for length, tree in iterator:
                yield length, tree
        else:
            trees_covered = 0
            j = 0
            # TODO this is ugly. Update the alg so we don't need this
            # bracketing.
            bp = [0] + breakpoints + [self.get_num_loci()]
            for length, tree in iterator:
                trees_covered += length
                while bp[j] < trees_covered:
                    j += 1
                    yield bp[j] - bp[j - 1], tree

    def dump(self, path, zlib_compression=False):
        """
        Writes the tree sequence to the specified file path.

        :param str path: The file path to write the TreeSequence to.
        :param bool zlib_compression: If True, use HDF5's native
            compression when storing the data leading to smaller
            file size. When loading, data will be decompressed
            transparently, but load times will be significantly slower.
        """
        self._ll_tree_sequence.dump(path, zlib_compression)

    def get_sample_size(self):
        """
        Returns the sample size for this tree sequence. This is the number
        of leaf nodes in each tree.

        :return: The number of leaf nodes in the tree sequence.
        :rtype: int
        """
        return self._ll_tree_sequence.get_sample_size()

    def get_num_loci(self):
        """
        Returns the number of loci in this tree sequence. This defined the
        genomic scaler over which tree coordinates are defined. Given a
        tree sequence with :math:`m` loci, the constituent trees will be
        defined over the half-closed interval :math:`(0, m])`. Each tree
        then covers some subset of this interval --- see
        :meth:`msprime.SparseTree.get_interval` for details.

        :return: The number of discrete non-recombining loci in this tree
            sequence.
        :rtype: int
        """
        return self._ll_tree_sequence.get_num_loci()

    def get_num_records(self):
        """
        Returns the number of coalescence records in this tree sequence.
        See the :meth:`.records` method for details on these objects.

        :return: The number of coalescence records defining this tree
            sequence.
        :rtype: int
        """
        return self._ll_tree_sequence.get_num_records()

    def get_num_mutations(self):
        """
        Returns the number of mutations in this tree sequence. See
        the :meth:`msprime.TreeSequence.mutations` method for details on how
        mutations are defined.

        :return: The number of mutations in this tree sequence.
        :rtype: int
        """
        return self._ll_tree_sequence.get_num_mutations()

    def get_num_nodes(self):
        """
        Returns the number of nodes in this tree sequence. This is the
        largest value :math:`u` such that `u` is a node in any of the
        constituent trees.

        :return: The total number of nodes in this tree sequence.
        :rtype: int
        """
        return self._ll_tree_sequence.get_num_nodes()

    def records(self):
        """
        Returns an iterator over the coalescence records in this tree
        sequence in time-sorted order. Each record is a tuple
        :math:`(l, r, u, c, t)` defining the assignment of a tree node
        across an interval. The range of this record is as the half-open
        genomic interval :math:`[l, r)`, such that it applies to all
        positions :math:`l \leq x < r`. Each record represents the
        assignment of a pair of children :math:`c` to a parent
        :math:`u`. This assignment happens at time :math:`t` in
        coalescent units.

        :return: An iterator of all :math:`(l, r, u, c, t)` tuples defining
            the coalescence records in this tree sequence.
        :rtype: iter
        """
        for j in range(self.get_num_records()):
            yield self._ll_tree_sequence.get_record(j)

    def diffs(self):
        """
        Returns an iterator over the differences between adjacent trees in this
        tree sequence. Each diff returned by this method is a tuple of the form
        `(length, records_out, records_in)`. The `length` is the length of the
        genomic interval covered by the current tree, and is equivalent to the
        value returned by :meth:`msprime.SparseTree.get_length`. The
        `records_out` value is list of :math:`(u, c, t)` tuples, and
        corresponds to the coalescence records that have been invalidated by
        moving to the current tree.  As in the :meth:`.records` method,
        :math:`u` is a tree node, :math:`c` is a tuple containing its children,
        and :math:`t` is the time the event occurred.  These records are
        returned in time-decreasing order, such that the record affecting the
        highest parts of the tree (i.e., closest to the root) are returned
        first.  The `records_in` value is also a list of :math:`(u, c, t)`
        tuples, and these describe the records that must be applied to create
        the tree covering the current interval. These records are returned in
        time-increasing order, such that the records affecting the lowest parts
        of the tree (i.e., closest to the leaves) are returned first.

        :return: An iterator over the diffs between adjacent trees in this
            tree sequence.
        :rtype: iter
        """
        return _msprime.TreeDiffIterator(self._ll_tree_sequence)

    def mutations(self):
        """
        Returns an iterator over the mutations in this tree sequence. Each
        mutation is represented as a tuple (position, node), and mutations
        returned in increasing order of position.

        :return: The mutations in this tree sequence.
        :rtype: iter
        """
        return iter(self._ll_tree_sequence.get_mutations())

    def trees(self, tracked_leaves=[]):
        """
        Returns an iterator over the trees in this tree sequence. Each value
        returned in this iterator is an instance of
        :class:`msprime.SparseTree`.  The ``tracked_leaves`` parameter can be
        used to efficiently count the number of leaves in a given set that
        exist in a particular subtree using the
        :meth:`msprime.SparseTree.get_num_tracked_leaves` method.

        :warning: Do not store the results of this iterator in a list!
           For performance reasons, the same underlying object is used
           for every tree returned which will most likely lead to unexpected
           behaviour.

        :param list tracked_leaves: The list of leaves to be tracked and
            counted using the
            :meth:`msprime.SparseTree.get_num_tracked_leaves` method.
        :return: An iterator over the sparse trees in this tree sequence.
        :rtype: iter
        """
        ll_sparse_tree = _msprime.SparseTree(
            self._ll_tree_sequence, tracked_leaves)
        iterator = _msprime.SparseTreeIterator(
            self._ll_tree_sequence, ll_sparse_tree)
        sparse_tree = SparseTree(ll_sparse_tree)
        for _ in iterator:
            yield sparse_tree

    def haplotypes(self):
        """
        Returns an iterator over the haplotypes resulting from the trees
        and mutations in this tree sequence as a string of '1's and '0's.
        The iterator returns a total of :math:`n` strings, each of which
        contains :math:`s` characters (:math:`n` is the sample size
        returned by :meth:`msprime.TreeSequence.get_sample_size` and
        :math:`s` is the number of mutations returned by
        :meth:`msprime.TreeSequence.get_num_mutations`). The first
        string returned is the haplotype for sample `1`, and so on.

        :return: An iterator over the haplotype strings for the samples in
            this tree sequence.
        :rtype: iter
        """
        return HaplotypeGenerator(self).haplotypes()

    def generate_mutations(self, scaled_mutation_rate, random_seed=None):
        """
        Generates mutation according to the infinite sites model. This
        method over-writes any existing mutations stored in the tree
        sequence.

        :param float scaled_mutation_rate: The rate of mutation
            per locus per :math:`4N_0` generations.
        :param int random_seed: The random seed to use when generating
            mutations.
        """
        seed = random_seed
        if random_seed is None:
            seed = random.randint(0, 2**31)
        self._ll_tree_sequence.generate_mutations(scaled_mutation_rate, seed)

    def set_mutations(self, mutations):
        """
        Sets the mutations in this tree sequence to the specified list of
        `(position, node)` tuples.  Each entry in the list must be a tuple of
        the form :math:`(x, u)`, where :math:`x` is a floating point value
        defining a genomic position and :math:`u` is an integer defining a tree
        node. A genomic position :math:`x` must satisfy :math:`0 \leq x < m`
        where :math:`m` is the number of loci (see :meth:`.get_num_loci`). A
        node :math:`u` must satisfy :math:`0 < u \leq N` where :math:`N` is the
        largest valued node in the tree sequence (see :meth:`.get_num_nodes`).

        :param list mutations: The list of mutations to be assigned to this
            tree sequence.
        """
        self._ll_tree_sequence.set_mutations(mutations)

    def get_pairwise_diversity(self):
        """
        Returns the value of pi, the pairwise nucleotide site diversity.

        :return: The pairwise nucleotide site diversity.
        :rtype: iter
        """
        pi = 0
        n = self.get_sample_size()
        denom = n * (n - 1) / 2
        for t in self.trees():
            for _, node in t.mutations():
                k = t.get_num_leaves(node)
                pi += k * (n - k) / denom
        return pi


class HaplotypeGenerator(object):

    def __init__(self, tree_sequence):
        self._tree_sequence = tree_sequence
        ts = self._tree_sequence.get_ll_tree_sequence()
        self._ll_haplotype_generator = _msprime.HaplotypeGenerator(ts)

    def get_haplotype(self, sample_id):
        return self._ll_haplotype_generator.get_haplotype(sample_id)

    def haplotypes(self):
        for j in range(1, self._tree_sequence.get_sample_size() + 1):
            yield self.get_haplotype(j)


class PopulationConfiguration(object):
    """
    TODO document.
    """
    def __init__(self, sample_size=0, initial_size=1.0, growth_rate=0.0):
        self.sample_size = sample_size
        self.initial_size = initial_size
        self.growth_rate = growth_rate

    def get_ll_representation(self):
        """
        Returns the low-level representation of this PopulationConfiguration.
        """
        return self.__dict__


class DemographicEvent(object):
    def __init__(self, type_, time):
        self.type = type_
        self.time = time


class GrowthRateChangeEvent(DemographicEvent):
    def __init__(self, time, growth_rate, population_id=-1):
        super(GrowthRateChangeEvent, self).__init__(
            "growth_rate_change", time)
        self.time = time
        self.growth_rate = growth_rate
        self.population_id = population_id

    def get_ll_representation(self, num_populations):
        return {
            "type": self.type,
            "time": self.time,
            "growth_rate": self.growth_rate,
            "population_id": self.population_id
        }

    def get_ms_arguments(self):
        if self.population_id == -1:
            return ["-eG", str(self.time), str(self.growth_rate)]
        else:
            return [
                "-eg", str(self.time), str(self.population_id + 1),
                str(self.growth_rate)]


class SizeChangeEvent(DemographicEvent):
    def __init__(self, time, size, population_id=-1):
        super(SizeChangeEvent, self).__init__("size_change", time)
        self.size = size
        self.population_id = population_id

    def get_ll_representation(self, num_populations):
        return {
            "type": self.type,
            "time": self.time,
            "size": self.size,
            "population_id": self.population_id
        }

    def get_ms_arguments(self):
        if self.population_id == -1:
            return ["-eN", str(self.time), str(self.size)]
        else:
            return [
                "-en", str(self.time), str(self.population_id + 1),
                str(self.growth_rate)]


class MigrationRateChangeEvent(DemographicEvent):
    def __init__(self, time, rate, matrix_index=None):
        super(MigrationRateChangeEvent, self).__init__(
            "migration_rate_change", time)
        self.rate = rate
        self.matrix_index = matrix_index

    def _convert_matrix_index(self, matrix_index):
        ret = -1
        if matrix_index is not None:
            ret = matrix_index[0] * self._num_populations + matrix_index[1]
        return ret

    def get_ll_representation(self, num_populations):
        matrix_index = -1
        if self.matrix_index is not None:
            matrix_index = (
                self.matrix_index[0] * num_populations + self.matrix_index[1])
        return {
            "type": self.type,
            "time": self.time,
            "migration_rate": self.rate,
            "matrix_index": matrix_index
        }

    def get_ms_arguments(self):
        raise NotImplemented()


class MassMigrationEvent(DemographicEvent):
    def __init__(self, time, source, destination, proportion):
        super(MassMigrationEvent, self).__init__("mass_migration", time)
        self.source = source
        self.destination = destination
        self.proportion = proportion

    def get_ll_representation(self, num_populations):
        return {
            "type": self.type,
            "time": self.time,
            "source": self.source,
            "destination": self.destination,
            "proportion": self.proportion
        }

    def get_ms_arguments(self):
        raise NotImplemented()


##############################
#
# Deprecated interface for specifying population demography
#
##############################


class PopulationModel(object):
    """
    Superclass of simulation population models.
    """
    def __init__(self, start_time):
        self.start_time = start_time


class ConstantPopulationModel(PopulationModel):
    """
    A population model in which the size of the population
    is a fixed multiple ``size`` of :math:`N_0` (the Wright-Fisher population
    size at time 0), which starts at the specified time.

    :param float start_time: The time (in coalescent units) at which
        this population model begins.
    :param float size: The size of the population under this model
        relative to :math:`N_0`.
    """
    def __init__(self, start_time, size):
        super(ConstantPopulationModel, self).__init__(start_time)
        self.size = size


class ExponentialPopulationModel(PopulationModel):
    """
    A population model in which the size is exponentially growing (or
    shrinking). If we have a :attr:`start_time` of :math:`s`, the population
    size at a time :math:`t` (measured in units of :math:`4N_0` generations)
    is :math:`N_s e^{\\alpha (t - s)}`, where :math:`N_s` is the population
    size at time :math:`s`.

    :param float start_time: The time (in coalescent units) at which
        this population model begins.
    :param float alpha: The exponential growth (or contraction) rate
        :math:`\\alpha`.
    """
    def __init__(self, start_time, alpha):
        super(ExponentialPopulationModel, self).__init__(start_time)
        self.alpha = alpha


def harmonic_number(n):
    """
    Returns the nth Harmonic number.
    """
    EulerGamma = 0.5772156649
    return math.log(n) + EulerGamma



"""
Simple client code for development purposes.
"""

from __future__ import print_function
from __future__ import division

import os
import math
import itertools

import msprime


def haplotype_example():
    tree_sequence = msprime.simulate(
        sample_size=10, num_loci=1000, scaled_recombination_rate=0.1,
        scaled_mutation_rate=0.01, random_seed=1)
    for h in tree_sequence.haplotypes():
        print(h)


def dump_example():
    tree_sequence = msprime.simulate(
        sample_size=10, num_loci=1000, scaled_recombination_rate=0.1,
        scaled_mutation_rate=0.01, random_seed=1)
    haplotypes = list(tree_sequence.haplotypes())
    tree_sequence.dump("example.hdf5")
    # Now, load another tree sequence instance from this file
    other_tree_sequence = msprime.load("example.hdf5")
    other_haplotypes = list(other_tree_sequence.haplotypes())
    assert haplotypes == other_haplotypes

def newick_example():
    tree_sequence = msprime.load("example.hdf5")
    with open("example.newick", "w") as f:
        iterator = tree_sequence.newick_trees(8)
        for l, ns in iterator:
            print("[{0}]".format(l), end="", file=f)
            print(ns, file=f)

def tree_example():
    tree_sequence = msprime.simulate(
        sample_size=10, num_loci=1000, scaled_recombination_rate=0.1,
        random_seed=1)
    for tree in tree_sequence.trees():
        print(tree)


def dump_file(filename):
    tree_sequence = msprime.load(filename)
    for r in tree_sequence.records():
        print(r)

class AnonymousObject():
    pass

def generate_trees(tree_sequence):
    N = tree_sequence.get_num_nodes() + 1
    M = tree_sequence.get_num_records()
    s = AnonymousObject()
    s.left = [0 for _ in range(M)]
    s.right = [0 for _ in range(M)]
    s.node = [0 for _ in range(M)]
    s.children = [None for _ in range(M)]
    s.time = [0 for _ in range(M)]
    for j, record in enumerate(tree_sequence.records()):
        s.left[j] = record[0]
        s.right[j] = record[1]
        s.node[j] = record[2]
        s.children[j] = record[3]
        s.time[j] = record[4]
    L = sorted(range(M), key=lambda j: (s.left[j], s.time[j]))
    R = sorted(range(M), key=lambda j: (s.right[j], -s.time[j]))
    t = AnonymousObject()
    t.parent = [0 for _ in range(N)]
    t.time = [0 for _ in range(N)]
    t.children = [None for _ in range(N)]
    t.left = 0
    t.right = s.right[R[0]]
    t.root = 0
    j = 0
    k = 0
    while True:
        while j < M and s.left[L[j]] == t.left:
            v = L[j]
            t.parent[s.children[v][0]] = s.node[v]
            t.parent[s.children[v][1]] = s.node[v]
            t.time[s.node[v]] = s.time[v]
            t.children[s.node[v]] = s.children[v]
            if s.node[v] > t.root:
                t.root = s.node[v]
            j += 1
        yield t
        if j == M:
            break
        while s.right[R[k]] == t.right:
            v = R[k]
            t.parent[s.children[v][0]] = 0
            t.parent[s.children[v][1]] = 0
            t.children[s.node[v]] = None
            t.time[s.node[v]] = 0
            if s.node[v] == t.root:
                t.root = max(s.children[v])
            k += 1
        t.left = t.right
        t.right = s.right[R[k]]

def tree_algorithm():
    tree_sequence = msprime.simulate(
        sample_size=40, num_loci=1000, scaled_recombination_rate=0.1,
        random_seed=1)
    for t1, t2 in zip(generate_trees(tree_sequence), tree_sequence.sparse_trees()):
        assert t1.parent == list(t2.parent)
        assert t1.time == list(t2.time)
        assert t1.left == t2.left
        assert t1.right == t2.right
        assert t1.root == t2.root

def large_example():
    msprime.simulate(10**6, 100 * 10**6, scaled_recombination_rate=0.001,
            random_seed=1, max_memory="10G")

def small_example():
    ts = msprime.simulate(5, 10, scaled_recombination_rate=0.1, random_seed=1)
    for sp in ts.trees():
        print(sp)
    ts.dump("example.hdf5")

def diffs_example():
    ts = msprime.simulate(5, 10, scaled_recombination_rate=0.1, random_seed=1)
    for length, records_out, records_in in ts.diffs():
        print(length, records_out, records_in)

def draw_tree():
    tree = msprime.simulate_tree(5, random_seed=1, scaled_mutation_rate=0.1)
    print(list(tree.mutations()))
    print(list(tree.nodes()))
    tree.draw("example.svg")

def draw_trees():
    # tree = msprime.simulate_trees(5, random_seed=1, scaled_mutation_rate=0.1)
    tree_sequence = msprime.simulate(
        10, 100, scaled_recombination_rate=1, random_seed=1)
    for j, tree in enumerate(tree_sequence.trees()):
        tree.draw("tmp__NOBACKUP__/example-{}.svg".format(j))

def large_leaf_count_example():
    n = 1000
    ts = msprime.simulate(n, 100000, scaled_recombination_rate=0.1, random_seed=1)
    num_trees = 0
    potential_nodes = []
    for tree, diff in itertools.izip(ts.trees(True), ts.diffs()):
        num_trees += 1
        _, _, records_in = diff
        # Go through all the nodes that are introduced in this tree and
        # see how many leaves they subtend.
        for node, _, _ in records_in:
            if tree.get_num_leaves(node) == 500:
                potential_nodes.append((node, tree.get_interval()))
    print(potential_nodes)
    print(num_trees)


def count_leaves(pi, n):
    nu = [0 for j in pi]
    for u in range(1, n + 1):
        v = u
        while v != 0:
            nu[v] += 1
            v = pi[v]
    return nu

def leaf_count_example():
    # This is an implementation of the algorithm to count the number
    # of leaves under each node.
    n = 20
    ts = msprime.simulate(n, 10000, scaled_recombination_rate=0.1, random_seed=1)
    pi = [0 for j in range(ts.get_num_nodes() + 1)]
    nu = [0 for j in range(ts.get_num_nodes() + 1)]
    for j in range(1, n + 1):
        nu[j] = 1
    for l, records_out, records_in in ts.diffs():
        for node, children, time in records_out:
            # print("out:", node, children)
            for c in children:
                pi[c] = 0
            leaves_lost = nu[node]
            u = node
            while u != 0:
                nu[u] -= leaves_lost
                # print("Setting nu[", u, "] = ", nu[u])
                u = pi[u]
        for node, children, time in records_in:
            # print("in:", node, children)
            num_leaves = 0
            for c in children:
                pi[c] = node
                num_leaves += nu[c]
            u = node
            while u != 0:
                nu[u] += num_leaves
                # print("setting nu[", u, "] = ", nu[u])
                u = pi[u]
        nup = count_leaves(pi, n)
        assert nup == nu
        # print("pi = ", pi)
        # print("nu = ", nu)
        # print("nup= ", nup)

class LeafSetNode(object):
    """
    Class representing a single leaf in the leaf set.
    """
    def __init__(self, value):
        self.value = value
        self.next = None
        self.prev = None

    def __str__(self):
        return str(self.value)

    def __repr__(self):
        return self.__str__()

def leaf_set_example():
    """
    Development for the leaf set algorithm.
    """
    n = 1000

    ts = msprime.simulate(n, 10000, scaled_recombination_rate=0.1, random_seed=1)
    head = [None for j in range(ts.get_num_nodes() + 1)]
    tail = [None for j in range(ts.get_num_nodes() + 1)]
    pi = [0 for j in range(ts.get_num_nodes() + 1)]
    chi = [None for j in range(ts.get_num_nodes() + 1)]
    for j in range(1, n + 1):
        set_node = LeafSetNode(j)
        head[j] = set_node
        tail[j] = set_node
    # print(ts)
    # for st in ts.trees():
    #     print(st)

    for st, (l, records_out, records_in) in itertools.izip(ts.trees(), ts.diffs()):
        for node, (c1, c2), time in records_out:
            print("out:", node, c1, c2)
            pi[c1] = 0
            pi[c2] = 0
            chi[node] = None
            # Break links in the leaf chain.
            tail[c1].next = None
            tail[c2].next = None
            # Clear out head and tail pointers.
            head[node] = None
            tail[node] = None

        for node, (c1, c2), time in records_in:
            print("in:", node, c1, c2)
            pi[c1] = node
            pi[c2] = node
            chi[node] = c1, c2
            u = node
            while u != 0:
                d1, d2 = chi[u]
                print("\tsettting head/tail", u, d1, d2)
                head[u] = head[d1]
                tail[u] = tail[d2]
                u = pi[u]

        for node, _, _ in records_in:
            u = node
            print("propagating", u)
            while u != 0:
                d1, d2 = chi[u]
                print("\tpropogating", u, d1, d2)
                print("\tbefore tail[d1] = ", tail[d1], "next = ", tail[d1].next)
                if tail[d1] is None:
                    break
                tail[d1].next = head[d2]
                print("\tafter  tail[d1] = ", tail[d1], "next = ", tail[d1].next)
                u = pi[u]


        print("node", "pi", "chi", "head", "tail", sep="\t")
        for j in range(ts.get_num_nodes() + 1):
            if st.get_parent(j) != 0 or j == st.get_root():
                print(
                    j, st.get_parent(j), st.get_children(j),
                    head[j], tail[j], sep="\t")
        # for u in st.nodes():
        for u in [st.get_root()]:
            leaves = list(st.leaves(u))
            leaf_list = []
            v = head[u]
            while v is not tail[u]:
                # print("\t", v.value, v.next)
                leaf_list.append(v.value)
                assert len(leaf_list) < n
                v = v.next
            leaf_list.append(v.value)
            # print(u, leaf_list, leaves, sep="\t")
            if leaf_list != leaves:
                print(leaf_list)
                print(leaves)
            assert leaf_list == leaves
        # print()

def allele_frequency_example():
    # n = 10000
    # ts = msprime.simulate(
    #     n, 100000, scaled_recombination_rate=0.1, scaled_mutation_rate=0.1,
    #     random_seed=1)
    ts = msprime.load("tmp__NOBACKUP__/gqt.hdf5")
    n = ts.get_sample_size()
    num_mutations = 0
    min_frequency = 0.0001
    num_trees = 0
    for tree in ts.trees():
        num_trees += 1
        for pos, node in tree.mutations():
            if tree.get_num_leaves(node) / n < min_frequency:
                num_mutations += 1
    print("num_mutatinos = ", num_mutations, "\t", num_mutations / ts.get_num_mutations())
    print("total_mutations = ", ts.get_num_mutations())
    print("num_trees = ", num_trees)

def write_ped(ts, num_cases, prefix):
    import numpy as np
    # Write the ped file.
    s = ts.get_num_mutations()
    indices = np.arange(0, s)
    with open(prefix + ".ped", "w") as f:
        for j, h in enumerate(ts.haplotypes(), 1):
            family_id = j
            ind_id = j
            paternal_id = 0
            maternal_id = 0
            sex = 1
            # 2 == case, 1 == control
            phenotype = 2 if j <= num_cases else 1
            print(
                family_id, ind_id, paternal_id, maternal_id, sex, phenotype,
                end=" ", file=f)
            # Neat trick for converting numerical strings to numpy array:
            # interpret data as raw chars, and subtrace ascii code.
            nph = np.fromstring(h, np.int8) - ord('0')
            # Plink genotypes are 1/2, not 0/1
            nph += 1
            genotypes = np.zeros(4 * s, dtype="S1")
            genotypes[indices * 4] = nph
            genotypes[indices * 4 + 1] = " "
            genotypes[indices * 4 + 2] = nph
            genotypes[indices * 4 + 3] = " "
            genotypes.tofile(f)
            # Remove trailing space
            f.seek(-1, os.SEEK_CUR)
            print(file=f)
    # Make a map file. We're using haploid data, so we put it on the
    # male X chromosome.
    with open(prefix + ".map", "w") as f:
        for j in range(ts.get_num_mutations()):
            print("X", "rs{}".format(j), 0, j + 1, file=f)



def gwas_example():
    # n = 100
    # ts = msprime.simulate(
    #     n, 1000, scaled_recombination_rate=0.1, scaled_mutation_rate=10,
    #     random_seed=1)
    ts = msprime.load("tmp__NOBACKUP__/gqt.hdf5")
    n = ts.get_sample_size()
    num_cases = n // 2
    # write_ped(ts, num_cases, "tmp__NOBACKUP__/test")
    # write_ped(ts, num_cases, "tmp__NOBACKUP__/plink/gqt")
    write_plink_assoc(ts, num_cases)

def write_plink_assoc(ts, num_cases):
    site = 0
    n = ts.get_sample_size()
    cases = list(range(1, num_cases + 1))
    for tree in ts.trees(cases):
        for pos, node in tree.mutations():
            num_leaves = tree.get_num_leaves(node)
            cases_with_mut = tree.get_num_tracked_leaves(node)
            controls_with_mut = tree.get_num_leaves(node) - cases_with_mut
            f_cases = cases_with_mut / num_cases
            f_controls = controls_with_mut / (n - num_cases)
            if num_leaves >= n / 2:
                # The mutation is the major allele
                a1 = 1
                a2 = 2
                fa = 1 - f_cases
                fu = 1 - f_controls
            else:
                # The mutation is the minor allele
                a1 = 2
                a2 = 1
                fa = f_cases
                fu = f_controls
            case_odds = fa / (1 - fa)
            control_odds = fu / (1 - fu)
            odds_ratio = "NA" if control_odds == 0 else case_odds / control_odds
            print(site + 1, a1, fa, fu, a2, odds_ratio, sep="\t")

def convert_haplotypes():
    import sys
    ts = msprime.load(sys.argv[1])
    ts.generate_mutations(0.0001, 1)
    print("Generated mutations", ts.get_num_mutations())
    c = 0
    for h in ts.haplotypes():
        c += 1
        # print(h)
    print("generated ", c, "haplotypes")

if __name__ == "__main__":
    # haplotype_example()
    # dump_example()
    # newick_example()
    # tree_example()
    # dump_file("example.hdf5")
    # tree_algorithm()
    # large_example()
    # draw_tree()
    # small_example()
    # draw_trees()
    # leaf_count_example()
    # large_leaf_count_example()
    # diffs_example()
    # leaf_set_example()
    # allele_frequency_example()
    # gwas_example()
    convert_haplotypes()
